<?php
/* $Id: cc_via_webpay.php,v 1.3 2009/05/13
 * ====================================================================
 * License:	GNU Lesser General Public License (LGPL)
 *
 * Copyright (c) 2005, 2006 Cameron Manderson (cameron@mink.net.au). 
 * All rights reserved.
 *
 * This file is part of the WebPay Payment Module intended for use in 
 * osCommerce.
 * 
 * osCommerce, Open Source E-Commerce Solutions
 * http://www.oscommerce.com - Copyright (c) 2003 osCommerce
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/**
 * Credit Card Payments through the Webteh Payment Gateway (WebPay) 
 * 
 * Used in the osCommerce package developed by Open Source E-Commerce Solutions
 * http://www.oscommerce.com Copyright (c) 2003 osCommerce
 * 
 * @author Damir Roso damir.roso@webteh.hr
 * @copyright Copyright &copy; 2007, Webteh d.o.o.
 */


define('MODULE_PAYMENT_WEBPAY_TEXT_TITLE', 'WEBPAY');
define('MODULE_PAYMENT_WEBPAY_TEXT_DESCRIPTION', 'Payment module for WebPay Internet Gateway Service');

class cc_via_webpay {
	// Variable Decleration
	var $code;
	var $title;
	var $description;
	var $enabled;	

	/**
	 * Instantiates a new object of class 'cc_via_webpay'
	 * Initiates the Variables required for the payment module
	 */
	function cc_via_webpay() {		
		// Specify Object operation parameters
		$this->code = 'cc_via_webpay';
		$this->title = (defined('MODULE_PAYMENT_WEBPAY_TEXT_TITLE') ? MODULE_PAYMENT_WEBPAY_TEXT_TITLE : 'Credit Card via WebPay'); // Change this for changing what appears on your site 
		$this->description = (defined('MODULE_PAYMENT_WEBPAY_TEXT_DESCRIPTION') ? MODULE_PAYMENT_WEBPAY_TEXT_DESCRIPTION : 'Payment module for WebPay Internet Gateway Service');
		$this->email_footer = (defined('MODULE_PAYMENT_WEBPAY_TEXT_EMAIL_FOOTER') ? MODULE_PAYMENT_WEBPAY_TEXT_EMAIL_FOOTER : '');
		
		/* Ensure that everything appears complete to enable this payment processor */
		if(defined('MODULE_PAYMENT_WEBPAY_STATUS') && strtoupper(MODULE_PAYMENT_WEBPAY_STATUS) == 'TRUE') {
			$merchantID = (defined('MODULE_PAYMENT_WEBPAY_MERCHANT_ID') ? MODULE_PAYMENT_WEBPAY_MERCHANT_ID : null);
			$accessCode = (defined('MODULE_PAYMENT_WEBPAY_ACCESS_CODE') ? MODULE_PAYMENT_WEBPAY_ACCESS_CODE : null); 
			$secretHash = "";
			if(!empty($merchantID) && !empty($accessCode)) {
				// Check for required under scenario
				if($this->isWebPayForm()) $this->enabled = (!empty($secretHash) ? true : false);
				else $this->enabled = true;
			} else $this->enabled = false;
		} else $this->enabled = false;
		
		$this->enabled = true;
		
		// Specify the form action url
		$this->sort_order = MODULE_PAYMENT_WEBPAY_SORT_ORDER;
	}
	
	/**
	 * Update the status of whether this payment module is available
	 * to check out with
	 * Reference: OSCommerce Payment Module
	 */
	function update_status() {
		/* Check whether the zones/geo_zones is valid */
		global $order;
		if (((int) MODULE_PAYMENT_WEBPAY_VALID_ZONE > 0)) {
			$check_flag = false;
			$check_query = tep_db_query("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_WEBPAY_VALID_ZONE . "' and zone_country_id = '".$order->delivery['country']['id']."' order by zone_id");
			while ($check = tep_db_fetch_array($check_query)) {
				if ($check['zone_id'] < 1) {
					$check_flag = true;
					break;
				}
				elseif ($check['zone_id'] == $order->delivery['zone_id']) {
					$check_flag = true;
					break;
				}
			}
			/* Set whether this should be valid or not */
			if ($check_flag == false) {
				$this->enabled = false;
			}
		}
	}

	/**
	 * Return with the Javascript validation for receiving CC details
	 * from the form (help make validation more efficient/effective)
	 * 
	 * @return String Validation to be used in the checkout form if required 
	 */
	function javascript_validation() {
		// If operating as Merchant Hosted Web Payment Pages
		if($this->isWebPayDirect()) {
			// Specify Javascript to validate the form elements
			// Credits: OSCommerce Javascript Validation from cc.php
			$js = '  if (payment_value == "' . $this->code . '") {' . "\n" .
			    '    var cc_owner = document.checkout_payment.cc_via_webpay_cc_owner.value;' . "\n" .
			    '    var cc_number = document.checkout_payment.cc_via_webpay_cc_number.value;' . "\n" .
			    '    if (cc_owner == "" || cc_owner.length < ' . CC_OWNER_MIN_LENGTH . ') {' . "\n" .
			    '      payment_error = payment_error + "' . MODULE_PAYMENT_WEBPAY_TEXT_JS_CC_OWNER . '";' . "\n" .
			    '      error = 1;' . "\n" .
			    '    }' . "\n" .
			    '    if (cc_number == "" || cc_number.length < ' . CC_NUMBER_MIN_LENGTH . ') {' . "\n" .
			    '      payment_error = payment_error + "' . MODULE_PAYMENT_WEBPAY_TEXT_JS_CC_NUMBER . '";' . "\n" .
			    '      error = 1;' . "\n" .
			    '    }' . "\n" .
			    '  }' . "\n";
			
			return $js;
		} else {
			// Else return empty String (No Validation required, just a selection)
			return '';
		}
	}


	/**
	 * Creates a selection array that defines the required fields 
	 * when providing this module as a potential payment method
	 * @return Array containing the module/form information to be used rendering
	 * 				 the form
	 */
	function selection() {
		// If operating as Merchant Hosted Web Payment Pages
		if($this->isWebPayDirect()) {
			// Specify the required form fields	
      		global $order;
			// Determine Expiry Month Field
			for ($i = 1; $i < 13; $i ++) {
				$expires_month[] = array ('id' => sprintf('%02d', $i), 'text' => strftime('%B', mktime(0, 0, 0, $i, 1, 2000)));
			}
			// Determine Expiry Year Field
			$today = getdate();
			for ($i = $today['year']; $i < $today['year'] + 10; $i ++) {
				$expires_year[] = array ('id' => strftime('%y', mktime(0, 0, 0, 1, 1, $i)), 'text' => strftime('%Y', mktime(0, 0, 0, 1, 1, $i)));
			}
			// Build the array with the fields
			// TODO: Use hidden Payment Attempt field
			$selection = array ('id' => $this->code, 'module' => $this->title, 'fields' => array (array ('title' => MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_OWNER, 'field' => tep_draw_input_field('cc_via_webpay_cc_owner', $order->billing['firstname'].' '.$order->billing['lastname'])), array ('title' => MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_NUMBER, 'field' => tep_draw_input_field('cc_via_webpay_cc_number')), array ('title' => MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_EXPIRES, 'field' => tep_draw_pull_down_menu('cc_via_webpay_cc_expires_month', $expires_month).'&nbsp;'.tep_draw_pull_down_menu('cc_via_webpay_cc_expires_year', $expires_year)), array ('title' => MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_CARD_SECURITY_CODE, 'field' => tep_draw_input_field('cc_via_webpay_cc_securitycode', '', 'size="4" maxlength="4"').'&nbsp;<small>'.MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_CARD_SECURITY_CODE_LOCATION.'</small>')));			
			return $selection;	
			
		} else {		
			// Else return the payment form as a selection
			// TODO: Include some information about the process
			$selection = array('id' => $this->code, 'module' => $this->title);
      		return $selection;
		}
	}


	/**
	 * Performs required validation to check and ensure that the 
	 * submitted form information appears correct before performming the 
	 * transaction through the confirmation screen.
	 * 
	 * Uses the cc_validation.php class to check the CC Number, expiry date
	 * and redirects an $error string containing the Friendly Error Message
	 * to the Checkout Payment screen
	 */
	function pre_confirmation_check() {
		// If operating as Merchant Hosted Web Payment Pages
		if($this->isWebPayDirect()) {
			global $HTTP_POST_VARS;			
			include (DIR_WS_CLASSES.'cc_validation.php');
			
			// Perform validation through the cc_validation class
			$cc_validation = new cc_validation();
			$result = $cc_validation->validate($HTTP_POST_VARS['cc_via_webpay_cc_number'], $HTTP_POST_VARS['cc_via_webpay_cc_expires_month'], $HTTP_POST_VARS['cc_via_webpay_cc_expires_year']);			
			$error = '';
			switch ($result) {
				case -1 :
					$error = sprintf(TEXT_CCVAL_ERROR_UNKNOWN_CARD, substr($cc_validation->cc_number, 0, 4));
					break;
				case -2 :
				case -3 :
				case -4 :
					$error = TEXT_CCVAL_ERROR_INVALID_DATE;
					break;
				case false :
					$error = TEXT_CCVAL_ERROR_INVALID_NUMBER;
					break;
			}

			// Redirect the user if contains an error			
			if (($result == false) || ($result < 1)) {
				$payment_error_return = 'payment_error='.$this->code.'&error='.urlencode($error).'&cc_owner='.urlencode($HTTP_POST_VARS['cc_via_webpay_cc_owner']).'&cc_via_webpay_cc_expires_month='.$HTTP_POST_VARS['cc_via_webpay_cc_expires_month'].'&cc_via_webpay_cc_expires_year='.$HTTP_POST_VARS['cc_via_webpay_cc_expires_year'];			
				tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, $payment_error_return, 'SSL', true, false));
			}
			
			// Treat the CC details to ensure they are ready to be sent to a payment gateway
			$this->cc_card_type = $cc_validation->cc_type;
			$this->cc_card_number = $cc_validation->cc_number;	
			$this->cc_expiry_month = $cc_validation->cc_expiry_month;
			$this->cc_expiry_year = $cc_validation->cc_expiry_year;
		}
		// Else allow filter through as no validation required
	}


	/**
	 * Return form fields displaying the confirmation information that will
	 * be used once they press the process button.
	 * 
	 * @return Array Confirmation elements array required to process the
	 * 				 form
	 */
	function confirmation() {
		// If operating as Direct
		if($this->isWebPayDirect()) {
			// Display the form details back to the user masking the credit card details
			global $HTTP_POST_VARS;
			$confirmation = array ('title' => $this->title.': '.$this->cc_card_type, 'fields' => array (array ('title' => MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_OWNER, 'field' => $HTTP_POST_VARS['cc_via_webpay_cc_owner']), array ('title' => MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_NUMBER, 'field' => substr($this->cc_card_number, 0, 4).str_repeat('X', (strlen($this->cc_card_number) - 8)).substr($this->cc_card_number, -4)), array ('title' => MODULE_PAYMENT_WEBPAY_TEXT_CREDIT_CARD_EXPIRES, 'field' => strftime('%B, %Y', mktime(0, 0, 0, $HTTP_POST_VARS['cc_via_webpay_cc_expires_month'], 1, '20'.$HTTP_POST_VARS['cc_via_webpay_cc_expires_year'])))));
			//return $confirmation;		
		} else {
			// TODO: Display information to the user based on the process
			$confirmation = array ('title' => $this->description);
		}
		return $confirmation;
	}


	/**
	 * Create the required Hidden Fields that contain the payment
	 * information to be used by the Payment Gateway (WebPay)
	 * 
	 * @return String Hidden Form Fields submitted to the action url
	 */
	function process_button() {
		global $HTTP_POST_VARS, $order;
		// If Submitting through the Direct Method
		if($this->isWebPayDirect()) {
			// Create all the required fields to be sent to the action url 
			// Required Fields
			$required_field_string = tep_draw_hidden_field('store_id', $this->getMerchantID()) .
									 //tep_draw_hidden_field('amount', intval((number_format($order->info['total'])))); 
									 tep_draw_hidden_field('card_details', $this->cc_card_number) .
									 tep_draw_hidden_field('cc_month', $this->cc_month) .
									 tep_draw_hidden_field('cc_year', $this->cc_year);

			// Optional Fields
			$optional_field_string = '';
			if(!empty($HTTP_POST_VARS['cc_via_webpay_cc_securitycode'])) {
				$optional_field_string .= tep_draw_hidden_field('card_verification_data' , $HTTP_POST_VARS['card_verification_data']);
			}				 

			$process_button_string = $required_field_string . $optional_field_string;
			return $process_button_string;			
		} else { // Else submitting directly to the Server
			// Required Fields
			$required_field_string = tep_draw_hidden_field('store_id', $this->getMerchantID()) .
									 //tep_draw_hidden_field('amount', intval((number_format($order->info['total'])))) 
									 tep_draw_hidden_field('language', $this->getLocale());
			
			// Optional Fields
			$optional_field_string = '';

			//$optional_field_string .= tep_draw_hidden_field('xxx', $this->xxx());

			// Ensure that Session Variables and Transaction Reference is attached
			$session_variables = '';
			$session_variables .= tep_draw_hidden_field('session_name',tep_session_name());
			$session_variables .= tep_draw_hidden_field('session_id', tep_session_id());
			//$session_variables .= tep_draw_hidden_field(tep_session_name(), tep_session_id()); // Value should be present by OSCommerce
			
			// Build the hidden fields
			$process_button_string = $required_field_string . $optional_field_string . $session_variables;
			return $process_button_string;			
		}
	}

	/**
	 * Perform additional required tasks before the processing
	 * @return boolean Function response
	 */
	function before_process() {	
		global $order,$currency,$languages_id,$cart;
		$verifiedHash = false;
		$responseArray = array();
		// Check the status of the process



		if(empty($_GET['order_number']) and empty($_POST['order_number'])){ // Payment hasn't been processed
			$verifiedHash = false;
			// Determine how to process the transaction
			
			//check configuration
			if ($this->getMerchantID()==''){
				echo "missing MerchantID";
				die();
			}
			if ($this->getAccessCode()==''){
				echo "missing AccessCode";
				die();
			}

			if($this->isWebPayDirect()){ // Process WebPay Direct
				echo 'not implemented';
				die();
			}	// END WebPay Direct			
						
			else { // Process WebPay Form
										// Perform the process
										//echo $this->getWebPayUrl();
										//die();
										echo '<html><head><title>WebPay redirect</title>
										<script language="Javascript">
										function OnLoadEvent(){document.form.submit();}
										</script>
										</head>
										<body OnLoad="OnLoadEvent();">
										redirecting to webpay ...<br/>';

										#foreach($order->customer as $key => $value) {
											#echo $key.' - '.$order->customer[$key];
									    #}

										echo '<form name="form" action="'.$this->getWebPayUrl().'" method="POST">
											<input type="hidden" name="cardholder_name" value="'.$order->customer['firstname'].'">
											<input type="hidden" name="cardholder_surname" value="'.$order->customer['lastname'].'">
											<input type="hidden" name="cardholder_address" value="'.$order->customer['street_address'].'">
											<input type="hidden" name="cardholder_city" value="'.$order->customer['city'].'">
											<input type="hidden" name="cardholder_zip_code" value="'.$order->customer['postcode'].'">
											<input type="hidden" name="cardholder_country" value="'.$order->billing['country']['title'].'">
											<input type="hidden" name="cardholder_phone" value="'.$order->customer['telephone'].'">
											<input type="hidden" name="cardholder_email" value="'.$order->customer['email_address'].'">
											<input type="hidden" name="hash" value="'.$this->getHash($order).'">
											<input type="hidden" name="store_id" value="'.$this->getMerchantID().'">
											<input type="hidden" name="require_complete" value="'.$this->getRequireComplete().'">
											<input type="hidden" name="order_number" value="'.$this->getOrderNumber().'">
											<input type="hidden" name="amount" value="'.(string)round($order->info['total'],2).'">
											<input type="hidden" name="currency" value="'.$this->getCurrency($currency).'">
											<input type="hidden" name="language" value="'.$this->getLocale($languages_id).'">
											<input type="hidden" name="mode" value="form">
											<input type="hidden" name="cart" value="'.$this->getCartContents($cart).'">
											<input type="hidden" name="cart_module" value="oscommerce2">
										<noscript><p>click to proceede to webpay</p><input type="submit"></noscript>
										</form></body></html>';

										//echo $required_field_string;
										//echo $webpayURL;
										die();						
				
 				 					} // END WebPay Form			
				} // END Payment hasn't been processed
				
				
				else { // START Payment processing with response from webpay

						// Assume resuming processing from check out process
						// Verify the secure hash to ensure that the communication wasn't altered or forged
						$secureHashResponse = $_GET['hash'];
						$order_numberResponse = $_GET['order_number'];
						unset($_GET['hash']);
						$responseArray = $_GET;

						if(!empty($secureHashResponse) && !empty($order_numberResponse)) { // verify incoming hash
						    // Check the SecureHashResponse against our generated hash to verify the key
						    if (strtoupper($secureHashResponse) == strtoupper($this->getHashForResponse())) {
						    	$verifiedHash = true;
						    	//die('Verified Successfully');
						    }
						} else {
							// Notify System Admin of the event that the SecretHash could not be verified as it wasn't present
							$errorMsg = (defined(MODULE_PAYMENT_WEBPAY_TEXT_ERROR_DESCRIPTION_SECURE_HASH_NOT_PRESENT) ? MODULE_PAYMENT_WEBPAY_TEXT_ERROR_DESCRIPTION_SECURE_HASH_NOT_PRESENT : "Secure Hash not present in Response from WebPay Server");		
							
							$this->sendNotifyEmail($errorMsg, $responseArray);
							// Redirect user with an error message saying that the response from the server was not complete and 
							// that we can not verify your payment. Please contact Admin to resolve	
							//tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . urlencode($errorMsg), 'SSL', true, false));
							tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=cc_via_webpay', 'SSL', true, false));
						}


						if($verifiedHash == false) {
							// Notify System Admin of the alteration or forgery by attempting to collect information from 
							// the user, such as their Customer ID, Order ID, IP etc, to double check records CC Merchant Account against 
							// incase the order was infact processed
							$errorMsg = (defined(MODULE_PAYMENT_WEBPAY_TEXT_ERROR_DESCRIPTION_SECURE_HASH_NOT_VALID) ? MODULE_PAYMENT_WEBPAY_TEXT_ERROR_DESCRIPTION_SECURE_HASH_NOT_VALID : "Secure Hash was not able to be verified against the Secret Hash, possible tampering with response");		
							$this->sendNotifyEmail($errorMsg, $responseArray);			

							// Redirect with a error message such as Tampering has occured with the response and to contact Admin to resolve	
							//tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . urlencode($errorMsg), 'SSL', true, false));					
							tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=cc_via_webpay', 'SSL', true, false));					
						} else {
							// Attempt to update some fields of the order in order to cache to database
							$order->info['cc_type'] = "";
							$order->info['cc_owner'] = "";
							$order->info['cc_number'] = "";
							$this->email_footer .= ""; //append to footer
						}

				} // END Payment processing with response from webpay
	} //end process_button


	/**
	 * Perform additional required tasks after the processing
	 * EG Mail details to store owner etc
	 * @return boolean Function response
	 */
	function after_process() {
		// TODO: Update Status of Order to Processing
		return false;	
	}

	/**
	 * Create an array that represents the possible module errors
	 * @return Array Errors that have arrived from HTTP 
	 */
	function get_error() {
		global $HTTP_GET_VARS;
		$error = array('title' => MODULE_PAYMENT_WEBPAY_TEXT_ERROR, 'error' => ((isset($HTTP_GET_VARS['error'])) ? stripslashes(urldecode($HTTP_GET_VARS['error'])) : MODULE_PAYMENT_WEBPAY_TEXT_ERROR_DESCRIPTION));
		return $error;
	}

	/**
	 * Checks whether the module has been installed 
	 * @return int 1 for installed
	 */
	function check() {
		if (!isset ($this->_check)) {
			$check_query = tep_db_query("select configuration_value from ".TABLE_CONFIGURATION." where configuration_key = 'MODULE_PAYMENT_WEBPAY_STATUS'");
			$this->_check = tep_db_num_rows($check_query);
		}
		return $this->_check;	
	}

	/**
	 * Performed when installing the WebPay Payment Processor
	 * SQL Inserts all the required configuration variables to be 
	 * assigned by the store owner in the administration screen
	 */
	function install() {
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable WebPay Payment Module', 'MODULE_PAYMENT_WEBPAY_STATUS', 'False', 'Do you want to accept payments through Webteh Internet Payment Service (WebPay)?', '6', '1', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('WebPay Test Simulator', 'MODULE_PAYMENT_WEBPAY_TEST_MODE', 'True', 'Do you wish to operate under the WebPay Test Simulator? It is important to recognise that the under this you will not be able to perform live transactions.', '6', '2', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");      
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('WebPay Merchant ID', 'MODULE_PAYMENT_WEBPAY_MERCHANT_ID', '', 'The unique Merchant ID assigned to you by your Payment Provider. (alphanumeric, upto 16 digits long)', '6', '3', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('WebPay Access Code', 'MODULE_PAYMENT_WEBPAY_ACCESS_CODE', '', 'The Access Code authenticates you on the WebPay Payment Server (alphanumeric, 8 digits long)', '6', '4', now())");
		
		
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Require Completetion for transactions', 'MODULE_PAYMENT_WEBPAY_REQUIRE_COMPLETE', 'False', 'Do you want to complete transactions (preauthorization)?', '6', '1', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
				
		
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('WebPay Direct Integration Model', 'MODULE_PAYMENT_WEBPAY_INTEGRATION_MODEL_DIRECT', 'False', 'Select TRUE to use the WebPay Direct Integration Model. Select FALSE to use the WebPay Form Integration Model', '6', '5', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");    
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Optional Notification Email Address', 'MODULE_PAYMENT_WEBPAY_EMAIL', '', 'You may optionally specify an email address to have payment error reported to such as Tampering Of Response and No Secure Hash Present notifications. By default if this is left empty the Site Admin will be notified', '6', '7', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Payment Zone', 'MODULE_PAYMENT_WEBPAY_VALID_ZONE', '0', 'If a zone is selected, only enable this payment method for that zone.', '6', '2', 'tep_get_zone_class_title', 'tep_cfg_pull_down_zone_classes(', now())");
		// Contribution from Richard Lee Friday 7th October Begin
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display', 'MODULE_PAYMENT_WEBPAY_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '2', now())");
		// Contribution End
	}

	/**
	 * Performs a SQL delete statement to remove all configuration 
	 * variables for this payment module
	 */
	function remove() {
		$keys = $this->keys(); 
		foreach($keys as $configurationKeys) {
			tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key = '" . $configurationKeys . "'");	
		}
	}

	/**
	 * Returns immediately with a listing of all the keys used 
	 * with this payment module
	 * @return Array All the defined variables used by this module
	 */
	function keys() {
		return array('MODULE_PAYMENT_WEBPAY_STATUS',
					 'MODULE_PAYMENT_WEBPAY_INTEGRATION_MODEL_DIRECT', 
					 'MODULE_PAYMENT_WEBPAY_TEST_MODE', 
					 'MODULE_PAYMENT_WEBPAY_MERCHANT_ID', 
					 'MODULE_PAYMENT_WEBPAY_ACCESS_CODE',
					 'MODULE_PAYMENT_WEBPAY_EMAIL', 
					 'MODULE_PAYMENT_WEBPAY_VALID_ZONE',
					 'MODULE_PAYMENT_WEBPAY_SORT_ORDER',
					 'MODULE_PAYMENT_WEBPAY_REQUIRE_COMPLETE'
					 );
	}

	/**
	 * Returns immediately with whether this is operating as a
	 * WebPay Direct model by referencing the administration const
	 * MODULE_PAYMENT_WEBPAY_INTEGRATION_MODEL_DIRECT
	 * @return boolean Check condition whether integration is WebPay Direct
	 */
	function isWebPayDirect() {
		if(defined('MODULE_PAYMENT_WEBPAY_INTEGRATION_MODEL_DIRECT')) {
			return(strtoupper(MODULE_PAYMENT_WEBPAY_INTEGRATION_MODEL_DIRECT) == 'TRUE');	
		}
		return false; // Default to WebPay Form
	}

	/**
	 * Returns immediately with whether this is operating as a
	 * WebPay Form Integration
	 */
	function isWebPayForm() {
		return(!$this->isWebPayDirect());
	}

	/**
	 * Returns immediately with whether this is operating in test mode
	 * should be configured in the administration panel.
	 * @return boolean Test Mode check
	 */
	function isTestMode() { 
		if(defined('MODULE_PAYMENT_WEBPAY_TEST_MODE')) {
			return(strtoupper(MODULE_PAYMENT_WEBPAY_TEST_MODE) == 'TRUE');	
		}
		return true; // Default to running test mode
	}

	/**
	 * Returns URI used for WebPay
	 * @return String used for the WebPay URI
	 */
	function getWebPayUrl() { 
		if($this->isTestMode()) {
	 		return 'https://test.webpay.cc/';
		} else {
			return 'https://secure.webteh.hr/';	
		}
	}

	/**
	 * Returns with the Merchant ID
	 * References defined variable MODULE_PAYMENT_WEBPAY_MERCHANT_ID
	 * @return String Treated Merchant ID
	 */
	function getMerchantID() {
		if($this->isTestMode()) {
	 		return '57';
		} else {
			if(defined('MODULE_PAYMENT_WEBPAY_MERCHANT_ID')) {
				return(trim(MODULE_PAYMENT_WEBPAY_MERCHANT_ID));
			}
			else {
				echo 'missing merchant id';
				die();
			}
		}
		
	}
	
	/**
	 * Returns immediately with the Access Key
	 * @return String Alphanumeric WebPay Account Access Key
	 */
	function getAccessCode() {
		if($this->isTestMode()) {
	 		return 'qwert123';
		} else {
			if(defined('MODULE_PAYMENT_WEBPAY_ACCESS_CODE')) {
				return(trim(MODULE_PAYMENT_WEBPAY_ACCESS_CODE));
				}
			else {
				echo 'missing access code';
				die();
			}	
		}
	}

	
	/**
	 * Returns customer_id to be associated with the Payment
	 * @return String customer_id
	 */
	function getCustomerId() { 
		global $customer_id;
		return($customer_id);
	}
	
	/**
	 * Returns with the selected currency to perform the payment under
	 * @return String currency
	 */
	function getCurrency($currency) { 
		return $currency;
	}
	
	/**
	 * Return with the Locale for this order
	 * @return String language code
	 */	
	function getLocale($languages_id=1) {
		$languages_query_raw = "select languages_id, name, code, image, directory, sort_order from " . TABLE_LANGUAGES . " where languages_id = ".$languages_id." order by sort_order";
		$languages_query = tep_db_query($languages_query_raw);
		$languages = tep_db_fetch_array($languages_query);
		return $languages['code'];
	}
	
	
	/**
	 * Returns with the selected require_complete to perform the payment under
	 * @return String require_complete
	 */
	function getRequireComplete() { 
		if(defined('MODULE_PAYMENT_WEBPAY_REQUIRE_COMPLETE')) {
			return strtolower(MODULE_PAYMENT_WEBPAY_REQUIRE_COMPLETE);	
		}		
	}
	
	/**
	 * Returns order_number to perform the payment under
	 * @return String order_number
	 */
	function getOrderNumber() { 
		$_SESSION['order_number'] = strftime('%Y%m%d%M%S').$this->getCustomerId(); //generate order_number using system time + customer_id
		return $_SESSION['order_number'];
	}
	
	/**
	 * Returns order_number to perform the payment under
	 * @return String hash
	 */
	function getHash($order) {
		return sha1($this->getAccessCode().$this->getOrderNumber().(string)round($order->info['total'],2).$this->getCurrency($currency));
	}
	
	
	
	/**
	 * Returns with the response Hash  
	 * @return String response Hash
	 */
	function getHashForResponse() { 
		return sha1($this->getAccessCode().$_SESSION['order_number']);
	}	
	
	/**
	 * Returns with the cart 
	 * @return String cart contents
	 */
	function getCartContents($cart) {
		$cart_contents_string = ''; 
		$products = $cart->get_products();
		for ($i=0, $n=sizeof($products); $i<$n; $i++) {
		$cart_contents_string .= $products[$i]['quantity'].'x ';
		$cart_contents_string .= $products[$i]['name'].', ';
		}
		return $cart_contents_string;
	}
	
	
	
	
	/**
	 * Return immediately with the WebPay version
	 * Optional: Specify MODULE_PAYMENT_WEBPAY_VIRTUAL_PAYMENT_CLIENT_VERSION
	 * to overwrite the default
	 * @return int WebPay Version 
	 */
	function getWebPayVersion() { 
		if(defined('MODULE_PAYMENT_WEBPAY_VIRTUAL_PAYMENT_CLIENT_VERSION')) {
			return(MODULE_PAYMENT_WEBPAY_VIRTUAL_PAYMENT_CLIENT_VERSION);
		}
		return 2.0; // Default to version 2.0
	}
	

	
	
	/**
	 * Returns with the Email to Notify of Transaction Errors
	 * @return String Email address of Account Admin
	 */
	function getNotifyEmailAddress() { 
		if(defined('MODULE_PAYMENT_WEBPAY_EMAIL') && tep_validate_email(MODULE_PAYMENT_WEBPAY_EMAIL)) {
			return MODULE_PAYMENT_WEBPAY_EMAIL;
		} else {
			if(defined('STORE_OWNER_EMAIL_ADDRESS') && tep_validate_email(STORE_OWNER_EMAIL_ADDRESS)) {
				return STORE_OWNER_EMAIL_ADDRESS;
			}	
		}
		return null; // No valid email specified
	}
	
	/**
	 * Sends an email alert to the registered WEBPAY Admins email address
	 * @param String $error Verbose Error
	 */
	function sendNotifyEmail($error, &$responseArray) {
		global $order;
		$notifyAddress = $this->getNotifyEmailAddress();
		if (!empty($notifyAddress)) {
			$subject = $error;
			$message = (defined('MODULE_PAYMENT_WEBPAY_TEXT_ERROR_EMAIL_INTRO') ? MODULE_PAYMENT_WEBPAY_TEXT_ERROR_EMAIL_INTRO : 'Error:') . "\n\n";
			$message .= $error . "\n\n";
			$message .= (defined('MODULE_PAYMENT_WEBPAY_TEXT_ERROR_EMAIL_INTRO_DETAILS') ? MODULE_PAYMENT_WEBPAY_TEXT_ERROR_EMAIL_INTRO_DETAILS : 'Payment Details') . "\n\n";
			$message .= "WEBPAY:\n";
			// Ensure that accesscode etc is ommitted
			foreach($responseArray as $key => $value) $message .= $key . '=' . $value . "\n";	
			$message .= "\n\nORDER CUSTOMER:\n";				
			foreach($order->customer as $key => $value) $message .= $key . '='. $value . "\n";
			$message .= "\n\nSERVER VARS:\n";				
			foreach($_SERVER as $key => $value) $message .= $key . '='. $value . "\n";						
			tep_mail('', $notifyAddress, $subject, $message, STORE_OWNER, STORE_OWNER_EMAIL_ADDRESS, '');
		}			
	}
}
?>